// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// This program generates the time series of concetration and inequality
// This version Jan 25, 2022
// Serdar Ozkan and Sergio Salgado
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

clear all
set more off
set type double
 
// You should change the below directory. 

global maindir =""

// Do not make change from here on. Contact Ozkan/Salgado if changes are needed. 
do "$maindir/do/0_Initialize.do"

// Create folder for output and log-file
global outfolder=c(current_date)
global outfolder="$outfolder Inequality"
capture noisily mkdir "$maindir${sep}out${sep}$outfolder"
capture log close
capture noisily log using "$maindir${sep}log${sep}$outfolder.log", replace

// Cd to the output file, create the program for moments, and load base sample.
cd "$maindir${sep}out${sep}$outfolder"
do "$maindir${sep}do${sep}myprogs.do"		

// Defines the number of points in the Kernel Density Estimator
global kpoints =  400

// Loop over the years
timer clear 1
timer on 1

foreach yr of numlist $yrlist{
	disp("")
	disp("------------------------------")
	disp("Working in year `yr'")
	disp("------------------------------")
// 	local yr = 2000			// For checking bugs
	// Define some variables to calculate
	local moreearn = ""
	if `yr' <= ${yrlast} - 1{
		local yrp = `yr' + 1
		local moreearn = "labor`yrp'"
	}
	if `yr' <= ${yrlast} - 5{
		local yrp = `yr' + 5
		local moreearn = "`moreearn' labor`yrp'"
	}
	
	// Load data 
	if inlist(`yr',${perm3yrlist}){
		use  male yob educ labor`yr' logearn`yr' researn`yr' permearn`yr' `moreearn' using ///
		"$maindir${sep}dta${sep}master_sample.dta" if labor`yr'~=. , clear   
	}
	else{
		use  male yob educ labor`yr' logearn`yr' researn`yr' `moreearn' using ///
		"$maindir${sep}dta${sep}master_sample.dta" if labor`yr'~=. , clear   
	}
	
	// Create year
	gen year=`yr'
	
	// Create age and restrict to CS sample
	gen age = `yr'-yob+1
	qui: drop if age<${begin_age} | age>${end_age}
	
	// Create the alternative age/cohort for plots
	qui: sum yob
	local minyob = r(min) 		
	gen yob2 = yob + mod(`minyob',${mergecohort}) - mod(yob,${mergecohort})
	
	gen agealt = `yr'-yob2+1
	qui: drop if agealt<${begin_age} | agealt>${end_age}
	
	// Drop if log earnings does not exist
	qui: drop if logearn==.
	
	// Gen earnings adjusted by the real min income
	gen earn = labor`yr' if labor`yr'>=rmininc[`yr'-${yrfirst}+1,1]
	
	// Age group 
	gen agegp = . 
	replace agegp = 1 if age <= 34 & agegp == .
	replace agegp = 2 if age <= 44 & agegp == .
	replace agegp = 3 if age <= 55 & agegp == .

	
	// A1. Moments of log earnings
	// Calculate cross sectional moments for year `yr'
	
	bymysum "logearn" "L_" "_`yr'" "year"
	
	bymyPCT "logearn" "L_" "_`yr'" "year"
	
	// Calculate cross sectional moments for year `yr' within heterogeneity groups
	foreach  vv in $hetgroup "male agealt"{ 
		local suf=subinstr("`vv'"," ","",.)
		
		bymysum "logearn" "L_" "_`suf'`yr'" "year `vv'"
	
		bymyPCT "logearn" "L_" "_`suf'`yr'" "year `vv'"
	}
	
	// A2. Moments of Residuals
	bymysum "researn" "L_" "_`yr'" "year"
	
	bymyPCT "researn" "L_" "_`yr'" "year"
	
	// Calculate cross sectional moments for year `yr' within heterogeneity groups
	foreach  vv in $hetgroup "male agealt"{ 
		local suf=subinstr("`vv'"," ","",.)
		
		bymysum "researn" "L_" "_`suf'`yr'" "year `vv'"
	
		bymyPCT "researn" "L_" "_`suf'`yr'" "year `vv'"
	}
	
	
	// B4. Moments of Permanent Income
	if inlist(`yr',${perm3yrlist}){
		bymysum "permearn" "L_" "_`yr'" "year"
	
		bymyPCT "permearn" "L_" "_`yr'" "year"
		
		foreach  vv in $hetgroup "male agealt"{ 
			local suf=subinstr("`vv'"," ","",.)
		
			bymysum "permearn" "L_" "_`suf'`yr'" "year `vv'"
	
			bymyPCT "permearn" "L_" "_`suf'`yr'" "year `vv'"
		}
	}
	
	
	// Calculate Empirical Density 
	if mod(`yr',${kyear}) == 0 {
		bymyKDN "logearn" "L_" "${kpoints}" "`yr'"
		
		bymyKDNmale "logearn" "L_" "${kpoints}" "`yr'"		
	}
	
	// Calculate measures of concentration
		bymyCNT "earn" "L_" "`yr'" 
		
	// Calculate the tail coefficients and shares of earnings	
		bymyRAT "earn" "RI_" "`yr'"
		
		forvalues mm = 0/1{
		*Gender
		preserve 
		keep if male == `mm'	
			bymyRAT "earn" "RI_male`mm'_" "`yr'"
		restore 
		
		*Gender age
		levelsof(agegp) if male == `mm', local(agp)		
		foreach aa of local agp{
		preserve 
			keep if male == `mm'
			keep if agegp == `aa'	
			bymyRAT "earn" "RI_male`mm'age`aa'_" "`yr'"
		restore
		} 
		}
		
		
	// Calculate measure of concentration within heterogeneity groups
	
		qui{
		*Male & Female
		forvalues mm = 0/1{
			*Over all
			preserve
			keep if male == `mm'
			bymyCNT "earn" "L_male`mm'" "`yr'" 
			restore 
			
			*by Educ
			levelsof(educ) if male == `mm', local(edu)
			foreach ee of local edu{
				preserve
				keep if male == `mm'
				keep if educ == `ee'
				bymyCNT "earn" "L_male`mm'educ`ee'" "`yr'" 
				restore
			}
			
			*By age
			levelsof(agegp) if male == `mm', local(agp)
			foreach aa of local agp{
				preserve
				keep if male == `mm'
				keep if agegp == `aa'
				bymyCNT "earn" "L_male`mm'age`aa'" "`yr'" 
				restore
			}	
		}
		*Education groups
		levelsof(educ), local(edu)
		foreach ee of local edu{
			preserve
			keep if educ == `ee'
			bymyCNT "earn" "L_educ`ee'" "`yr'" 
			restore
		}
		
		*By age
		levelsof(agegp), local(agp)
		foreach aa of local agp{
			preserve
			keep if agegp == `aa'
			bymyCNT "earn" "L_age`aa'" "`yr'" 
			restore
		}
		
		}	// END of qui statement
		
} // END of loop over years

// Calculate moments of Residuals Earnings with Education Controls
foreach yr of numlist $yrlist{

        // Load data
        use researne`yr' using "$maindir${sep}dta${sep}researne.dta", clear

        // Create year
        gen year=`yr'

        // Moments of Residuals by Education
        bymysum "researne" "L_" "_`yr'" "year"

        bymyPCT "researne" "L_" "_`yr'" "year"
}

// Calculate autocorrelation moments
// This is based on Luigi's codes

foreach yr of numlist $yrlist{
   use personid male yob researn${yrfirst}-researn${yrlast} if researn`yr' != . using ///
           "$maindir${sep}dta${sep}master_sample.dta" , clear

   gen year = `yr'

   gen age = year - yob + 1
   
   keep if age>=${begin_age} & age<= ${end_age}
   
   rename researn`yr' u
   
   gen u1=.
   gen u2=.
   gen u3=.
   gen u4=.
   gen u5=.
   gen u0u1=.
   gen u0u2=.
   gen u0u3=.
   gen u0u4=.
   gen u0u5=.
   
   gen du=.
   gen du1=.
   gen du2=.
   gen du3=.
   gen du4=.
   gen du5=.
   gen du0du1=.
   gen du0du2=.
   gen du0du3=.
   gen du0du4=.
   gen du0du5=.
   
   // lag 1 //
   local yl1=`yr' - 1
   if `yl1' >= ${yrfirst} {
      replace u1=researn`yl1'
      replace u0u1=u*u1
      replace du=u-u1
   }
   
   // lag 2 //
   local yl2=`yr' - 2
   if `yl2' >= ${yrfirst} {
      replace u2=researn`yl2'
      replace u0u2=u*u2
      replace du1=u1-u2
      replace du0du1=du*du1
   }
   
   // lag 3 //
   local yl3=`yr' - 3
   if `yl3' >= ${yrfirst} {
      replace u3=researn`yl3'
      replace u0u3=u*u3
      replace du2=u2-u3
      replace du0du2=du*du2
   }
   
   // lag 4 //
   local yl4=`yr' - 4
   if `yl4' >= ${yrfirst} {
      replace u4=researn`yl4'
      replace u0u4=u*u4
      replace du3=u3-u4
      replace du0du3=du*du3
   }
   
   // lag 5 //
   local yl5=`yr' - 5
   if `yl5' >= ${yrfirst} {
      replace u5=researn`yl5'
      replace u0u5=u*u5
      replace du4=u4-u5
      replace du0du4=du*du4
   }
   
   // lag 6 //
   local yl6=`yr' - 6
   if `yl6' >= ${yrfirst} {
      replace du5=u5-researn`yl6'
      replace du0du5=du*du5
   }
   
   keep u0u* du0du* du u u1-u5 du1-du5 year year male age
   compress
   
   preserve
   collapse u0u* du0du* (sd) du u u1-u5 du1-du5,by(year)
   gen ac_researn_1=u0u1/(u*u1)
   gen ac_researn_2=u0u2/(u*u2)
   gen ac_researn_3=u0u3/(u*u3)
   gen ac_researn_4=u0u4/(u*u4)
   gen ac_researn_5=u0u5/(u*u5)
   gen ac_dresearn_1=du0du1/(du*du1)
   gen ac_dresearn_2=du0du2/(du*du2)
   gen ac_dresearn_3=du0du3/(du*du3)
   gen ac_dresearn_4=du0du4/(du*du4)
   gen ac_dresearn_5=du0du5/(du*du5)
   save temp_lev_year_`yr',replace
   
   
   restore, preserve
   collapse u0u* du0du* (sd) du u u1-u5 du1-du5,by(year male)
   gen ac_researn_1=u0u1/(u*u1)
   gen ac_researn_2=u0u2/(u*u2)
   gen ac_researn_3=u0u3/(u*u3)
   gen ac_researn_4=u0u4/(u*u4)
   gen ac_researn_5=u0u5/(u*u5)
   gen ac_dresearn_1=du0du1/(du*du1)
   gen ac_dresearn_2=du0du2/(du*du2)
   gen ac_dresearn_3=du0du3/(du*du3)
   gen ac_dresearn_4=du0du4/(du*du4)
   gen ac_dresearn_5=du0du5/(du*du5)
   save temp_lev_gender_`yr',replace
   
   
   restore, preserve
   collapse u0u* du0du* (sd) du u u1-u5 du1-du5,by(year age)
   gen ac_researn_1=u0u1/(u*u1)
   gen ac_researn_2=u0u2/(u*u2)
   gen ac_researn_3=u0u3/(u*u3)
   gen ac_researn_4=u0u4/(u*u4)
   gen ac_researn_5=u0u5/(u*u5)
   gen ac_dresearn_1=du0du1/(du*du1)
   gen ac_dresearn_2=du0du2/(du*du2)
   gen ac_dresearn_3=du0du3/(du*du3)
   gen ac_dresearn_4=du0du4/(du*du4)
   gen ac_dresearn_5=du0du5/(du*du5)
   save temp_lev_age_`yr',replace
   
   
   restore, preserve
   collapse u0u* du0du* (sd) du u u1-u5 du1-du5,by(year male age)
   gen ac_researn_1=u0u1/(u*u1)
   gen ac_researn_2=u0u2/(u*u2)
   gen ac_researn_3=u0u3/(u*u3)
   gen ac_researn_4=u0u4/(u*u4)
   gen ac_researn_5=u0u5/(u*u5)
   gen ac_dresearn_1=du0du1/(du*du1)
   gen ac_dresearn_2=du0du2/(du*du2)
   gen ac_dresearn_3=du0du3/(du*du3)
   gen ac_dresearn_4=du0du4/(du*du4)
   gen ac_dresearn_5=du0du5/(du*du5)
   save temp_lev_age_gender_`yr',replace
   
   
   restore, preserve
   replace age=2534 if age>=25 & age<=34
   replace age=3544 if age>=35 & age<=44
   replace age=4555 if age>=45 & age<=55
   collapse u0u* du0du* (sd) du u u1-u5 du1-du5,by(year age)
   gen ac_researn_1=u0u1/(u*u1)
   gen ac_researn_2=u0u2/(u*u2)
   gen ac_researn_3=u0u3/(u*u3)
   gen ac_researn_4=u0u4/(u*u4)
   gen ac_researn_5=u0u5/(u*u5)
   gen ac_dresearn_1=du0du1/(du*du1)
   gen ac_dresearn_2=du0du2/(du*du2)
   gen ac_dresearn_3=du0du3/(du*du3)
   gen ac_dresearn_4=du0du4/(du*du4)
   gen ac_dresearn_5=du0du5/(du*du5)
   save temp_lev_age1_`yr',replace
   
   
   restore, preserve
   replace age=2534 if age>=25 & age<=34
   replace age=3544 if age>=35 & age<=44
   replace age=4555 if age>=45 & age<=55
   collapse u0u* du0du* (sd) du u u1-u5 du1-du5,by(year male age)
   gen ac_researn_1=u0u1/(u*u1)
   gen ac_researn_2=u0u2/(u*u2)
   gen ac_researn_3=u0u3/(u*u3)
   gen ac_researn_4=u0u4/(u*u4)
   gen ac_researn_5=u0u5/(u*u5)
   gen ac_dresearn_1=du0du1/(du*du1)
   gen ac_dresearn_2=du0du2/(du*du2)
   gen ac_dresearn_3=du0du3/(du*du3)
   gen ac_dresearn_4=du0du4/(du*du4)
   gen ac_dresearn_5=du0du5/(du*du5)
   save temp_lev_age_gender1_`yr',replace
   
   
   restore, preserve
   replace age=2555 if age>=25 & age<=55
   collapse u0u* du0du* (sd) du u u1-u5 du1-du5,by(year age)
   gen ac_researn_1=u0u1/(u*u1)
   gen ac_researn_2=u0u2/(u*u2)
   gen ac_researn_3=u0u3/(u*u3)
   gen ac_researn_4=u0u4/(u*u4)
   gen ac_researn_5=u0u5/(u*u5)
   gen ac_dresearn_1=du0du1/(du*du1)
   gen ac_dresearn_2=du0du2/(du*du2)
   gen ac_dresearn_3=du0du3/(du*du3)
   gen ac_dresearn_4=du0du4/(du*du4)
   gen ac_dresearn_5=du0du5/(du*du5)
   save temp_lev_age2_`yr',replace
   
   
   restore
   replace age=2555 if age>=25 & age<=55
   collapse u0u* du0du* (sd) du u u1-u5 du1-du5,by(year male age)
   gen ac_researn_1=u0u1/(u*u1)
   gen ac_researn_2=u0u2/(u*u2)
   gen ac_researn_3=u0u3/(u*u3)
   gen ac_researn_4=u0u4/(u*u4)
   gen ac_researn_5=u0u5/(u*u5)
   gen ac_dresearn_1=du0du1/(du*du1)
   gen ac_dresearn_2=du0du2/(du*du2)
   gen ac_dresearn_3=du0du3/(du*du3)
   gen ac_dresearn_4=du0du4/(du*du4)
   gen ac_dresearn_5=du0du5/(du*du5)
   save temp_lev_age_gender2_`yr',replace
   
   clear
   u temp_lev_year_`yr',clear
   append using temp_lev_gender_`yr'
   append using temp_lev_age_`yr'
   append using temp_lev_age_gender_`yr'
   append using temp_lev_age1_`yr'
   append using temp_lev_age_gender1_`yr'
   append using temp_lev_age2_`yr'
   append using temp_lev_age_gender2_`yr'
   
   keep year age male ac_*
   g str3 country="${iso}"
   
   tostring age,replace
   replace age="25-55" if age=="."
   replace age="25-34" if age=="2534"
   replace age="35-44" if age=="3544"
   replace age="45-55" if age=="4555"
   
   g str12 gender="All genders"
   replace gender="Male" if male==1
   replace gender="Female" if male==0
   drop male
   save temporal_`yr', replace
   
   erase temp_lev_year_`yr'.dta
   erase temp_lev_gender_`yr'.dta
   erase temp_lev_age_`yr'.dta
   erase temp_lev_age_gender_`yr'.dta
   erase temp_lev_age1_`yr'.dta
   erase temp_lev_age_gender1_`yr'.dta
   erase temp_lev_age2_`yr'.dta
   erase temp_lev_age_gender2_`yr'.dta

} // END of loop over years   
